from roulette_russe.Scraper import Scraper
from linkedin_api import Linkedin
from linkedin_api.utils.helpers import get_id_from_urn
from urllib.parse import urlencode

class LinkedinEnhanced(Linkedin):
    def __init__(self, username, password):
        super(LinkedinEnhanced, self).__init__(username=username, password=password)

    def get_company_employees(self, public_id):
        filters = ["resultType->PEOPLE","currentCompany->"+public_id]
        params = {"filters": "List({})".format(",".join(filters))}
        data = self.search(params, limit=1000)

        results = []
        for item in data:
            if "publicIdentifier" not in item:
                continue
            results.append('https://www.linkedin.com/in/'+get_id_from_urn(item.get("targetUrn")))
        return results

    def get_company_open_jobs(self, id):
        filters = [""]

    def search(self, params, limit=-1, offset=0):
        print('OOOOOOOK')
        """
        Do a search.
        """
        count = Linkedin._MAX_SEARCH_COUNT
        if limit is None:
            limit = -1

        results = []
        while True:
            # when we're close to the limit, only fetch what we need to
            if limit > -1 and limit - len(results) < count:
                count = limit - len(results)
            default_params = {
                "count": str(count),
                "filters": "List()",
                "origin": "OTHER",
                "q": "all",
                "start": len(results) + offset,
                "queryContext": "List()",
            }
            default_params.update(params)

            res = self._fetch(
                f"/search/blended?{urlencode(default_params, safe='(),')}",
                headers={"accept": "application/vnd.linkedin.normalized+json+2.1"},
            )
            data = res.json()

            new_elements = []
            elements = data.get("data", {}).get("elements", [])
            for i in range(len(elements)):
                new_elements.extend(elements[i]["elements"])
                # not entirely sure what extendedElements generally refers to - keyword search gives back a single job?
                # new_elements.extend(data["data"]["elements"][i]["extendedElements"])
            results.extend(new_elements)

            # break the loop if we're done searching
            # NOTE: we could also check for the `total` returned in the response.
            # This is in data["data"]["paging"]["total"]
            if (
                    (
                            limit > -1 and len(results) >= limit
                    )  # if our results exceed set limit
                    or len(results) / count >= Linkedin._MAX_REPEATED_REQUESTS
            ) or len(new_elements) == 0:
                break

            self.logger.debug(f"results grew to {len(results)}")

        return results


class LinkedinScraper(Scraper):
    def __init__(self, input_config, output_datasets, username, password, rr=None):
        super(LinkedinScraper, self).__init__(input_config, output_datasets, rr)
        self.api = LinkedinEnhanced(username, password)


    def query_company_details(self, account):
        company = self.api.get_company(account)
        return {
            "account": account,
            "company_name": company['name'],
            "tagline": company['tagline'],
            "description": company['description'],
            "company_industries": list(map(lambda x: x['localizedName'], company['companyIndustries'])),
            "company_specialities": company['specialities'],
            "company_headquarters": company['headquarter']['country'],
            "company_logo": company['logo']['image']['com.linkedin.common.VectorImage']['rootUrl'],
            "company_type": company['companyType']['code'],
            "company_followers": company['followingInfo']['followerCount'],
            "page_url": company['url'],
            "staff_count": company['staffCount']
        }

    def extract_company_updates(self, o):
        mentions = []
        if hasattr(o['value']['com.linkedin.voyager.feed.render.UpdateV2'], 'commentary'):
            if hasattr(o['value']['com.linkedin.voyager.feed.render.UpdateV2']['commentary']['text'], 'attributes'):
                mentions = list(map(lambda x: x['miniProfile']['publicIdentifier'],
                                    o['value']['com.linkedin.voyager.feed.render.UpdateV2']['commentary']['text'][
                                        'attributes']))
        return {
            "reactions":
                o['value']['com.linkedin.voyager.feed.render.UpdateV2']['socialDetail']['totalSocialActivityCounts'][
                    'reactionTypeCounts'],
            "reactions_count":
                o['value']['com.linkedin.voyager.feed.render.UpdateV2']['socialDetail']['totalSocialActivityCounts'][
                    'reactionTypeCounts'],
            "url": o['permalink'],
            "mentions": mentions,
            "comments_count":
                o['value']['com.linkedin.voyager.feed.render.UpdateV2']['socialDetail']['totalSocialActivityCounts'][
                    'numComments'],
            "id": o['id']
        }

    def query_company_updates(self, account):
        company_updates = self.api.get_company_updates(account)
        return list(map(lambda x : self.extract_company_updates(x), company_updates))

    def query_company_employees(self, account):
        employees = self.api.get_company_employees(account)
        return employees

    def scrap(self):
        tmp_list_profiles = []
        tmp_list_posts = []
        tmp_list_employees = []
        for account in self.input_config['accounts']:
            profile_data = self.query_company_details(account)
            tmp_list_profiles.append(profile_data)
        self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list_profiles, ignore_index=True)
        if 'posts' in self.output_datasets:
            for account in self.input_config['accounts']:
                posts_data = self.query_company_updates(account)
                tmp_list_posts += list(posts_data)
            self.output_datasets['posts'] = self.output_datasets['posts'].append(tmp_list_posts, ignore_index=True)
        if 'employees' in self.output_datasets:
            for account in self.input_config['accounts']:
                employees_data = self.query_company_employees(account)
                tmp_list_employees.append({'account': account, 'followers': list(employees_data)})
            self.output_datasets['employees'] = self.output_datasets['employees'].append(tmp_list_employees,
                                                                                         ignore_index=True)

    def quality_check(self):
        pass



import requests
from bs4 import BeautifulSoup
from roulette_russe.Scraper import Scraper


class IndeedScraper(Scraper):
    def __init__(self, input_config, output_datasets, rr=None):
        super(IndeedScraper, self).__init__(input_config, output_datasets, rr)

    def _extract_features(self, soup, feature):
        divs = soup.findAll('a', {'class': 'cmp-CompactHeaderMenuItem-link cmp-u-noUnderline'})
        for div in divs:
            if feature in div['href']:
                if len(div.findAll('div', {'class': 'cmp-CompactHeaderMenuItem-count'})) == 1:
                    return div.findAll('div', {'class': 'cmp-CompactHeaderMenuItem-count'})[0].text
        trs = soup.findAll('tr', {"class": "cmp-RatingCategory cmp-RatingCategory--md"})
        for tr in trs:
            if len(tr.findAll('td')) == 3:
                if feature in tr.findAll('td')[2].text:
                    return tr.findAll('td')[0].text
        return '-'

    def _extract_ceoRating(self, soup):
        if len(soup.findAll('div', {"class": "cmp-CeoWidgetWithRating-percent"})) == 1:
            return soup.findAll('div', {"class": "cmp-CeoWidgetWithRating-percent"})[0].text
        return '-'

    def _extract_rating(self, soup):
        return float(soup.findAll('span', {'class': 'cmp-CompactHeaderCompanyRatings-value'})[0].text.replace(',','.'))

    def _extract_name(self, soup):
        return soup.findAll('span', {'class': 'cmp-CompactHeaderCompanyName'})[0].text

    def query_company_main_infos(self, link):
        r = requests.get(link, proxies=self.get_proxies()).text
        soup = BeautifulSoup(r, 'html.parser')
        return {
            'name': self._extract_name(soup),
            'rating': self._extract_rating(soup),
            'reviews_number': self._extract_features(soup, 'reviews'),
            'salaries_number': self._extract_features(soup, 'salaries'),
            'jobs_number': self._extract_features(soup, 'jobs'),
            'photos_number': self._extract_features(soup, 'photos'),
            'direction': self._extract_features(soup, 'Direction'),
            'culture_entreprise': self._extract_features(soup, 'Culture d\'entreprise'),
            'equilibre_pro_perso': self._extract_features(soup, 'Équilibre vie professionnelle / personnelle'),
            'salaire_avantages': self._extract_features(soup, 'Salaire/Avantages sociaux'),
            'securite_emploi': self._extract_features(soup, 'Sécurité de l\'emploi/Évolution de carrière'),
            'ceo_rating': self._extract_ceoRating(soup)
        }


    def scrap(self):
        tmp_list = []
        for link in self.input_config['links']:
            tmp_list.append(self.query_company_main_infos(link))
        self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list, ignore_index=True)

    def quality_check(self):
        pass


from roulette_russe.Scraper import Scraper
from facebook_scraper import get_posts
import requests
from bs4 import BeautifulSoup

class FacebookScraper(Scraper):
    def __init__(self, input_config, output_datasets, rr=None):
        super(FacebookScraper, self).__init__(input_config, output_datasets, rr)

    def query_posts(self, account):
        return get_posts(account=account,page_limit=20, options=[])

    def query_company_infos(self, account):
        r = requests.get('https://m.facebook.com/'+account+'/?locale2=fr_FR', headers={
            'User-Agent': 'Mozilla/5.0 (Linux; Android 7.0; SM-G892A Build/NRD90M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/67.0.3396.87 Mobile Safari/537.36'
        }, proxies=self.get_proxies()).text

        # Extract main infos
        soup = BeautifulSoup(r, 'html.parser')

        # Extract account name
        name = soup.findAll('div', {"id": 'u_0_d'})[0].text

        # Extract logo
        logo = soup.findAll('img', {"id": "u_0_5"})[0]['src']
        # Extract community
        r2 = requests.get('https://m.facebook.com/'+account+'/community/', headers={
            'User-Agent': 'Mozilla/5.0 (Linux; U; en-us; KFAPWI Build/JDQ39) AppleWebKit/535.19 (KHTML, like Gecko) Silk/3.13 Safari/535.19 Silk-Accelerated=true'
        }, proxies=self.get_proxies()).text
        soup2 = BeautifulSoup(r2, 'html.parser')
        number_of_likes = soup2.findAll('div', {"class": "bn"})[0].text
        number_of_followers = soup2.findAll('div', {"class": "bn"})[1].text

        return {
            'name': name,
            'logo': logo,
            'number_of_likes': number_of_likes,
            'number_of_followers': number_of_followers
        }

    def scrap(self):
        tmp_list_profiles = []
        tmp_list_posts = []
        for account in self.input_config['accounts']:
            profile_data = self.query_company_infos(account)
            tmp_list_profiles.append(profile_data)
        self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list_profiles, ignore_index=True)
        if 'posts' in self.output_datasets:
            for account in self.input_config['accounts']:
                self.output_datasets['posts'] = self.output_datasets['posts'].append(
                    list(self.query_posts(account)),
                    ignore_index=True)
        pass

    def quality_check(self):
        pass



import requests
from bs4 import BeautifulSoup
from roulette_russe.Scraper import Scraper


class FigaroInsidersScraper(Scraper):
    def __init__(self, input_config, output_datasets, rr=None):
        super(FigaroInsidersScraper, self).__init__(input_config, output_datasets, rr)

    def extract_feature(self, soup, feature):
        divs = soup.findAll('div', {'class': 'top-flop-item--rating'})
        for div in divs:
            if div.findAll('span')[0].text == feature:
                return div.findAll('small')[0].text
        return 0

    def _get_name(self, soup):
        return soup.findAll('div',{"class": "enterprise-header--information--details--title--name"})[0].findAll('h1')[0].text

    def _get_global_score(self, soup):
        return soup.findAll('div',{'class': 'enterprise-header--information--rating'})[0].findAll('span')[0].text

    def _get_number_of_ratings(self, soup):
        return soup.findAll('span', {'class': 'global-rating-stats--overall--count'})[0].findAll('b')[0].text

    def _get_positive_ratings(self, soup):
        return soup.findAll('div',{'class':'top-flop-full--summary__top'})[0].findAll('b')[0].text

    def _get_neutral_ratings(self, soup):
        return soup.findAll('div',{'class':'top-flop-full--summary__neutral'})[0].findAll('b')[0].text

    def _get_negative_ratings(self, soup):
        return soup.findAll('div',{'class':'top-flop-full--summary__flop'})[0].findAll('b')[0].text



    def query_company_infos(self, link):
        r = requests.get(link+'/notes', proxies=self.get_proxies())
        soup = BeautifulSoup(r, 'html.parser')
        return {
            'name': self._get_name(soup),
            'global_score': self._get_global_score(soup),
            'number_of_ratings': self._get_number_of_ratings(soup),
            'positive_ratings': self._get_positive_ratings(soup),
            'neutral_ratings': self._get_neutral_ratings(soup),
            'negative_ratings': self._get_negative_ratings(soup),
            'facilite_acces': self.extract_feature(soup, 'Facilité d\'accès'),
            'avantage_sociaux': self.extract_feature(soup, 'Avantages sociaux'),
            'atteinte_objectifs_entraprise': self.extract_feature(soup, 'Atteinte des objectifs de l\'entreprise'),
            'salaire': self.extract_feature(soup, 'Salaire'),
            'reconnaissance_manager': self.extract_feature(soup, 'Reconnaissance du manager'),
            'responsable_autonomie': self.extract_feature(soup, 'Responsabilité / Autonomie'),
            'espace_travail_equipements': self.extract_feature(soup, 'Espace de travail et équipements'),
            'ambiance_travail': self.extract_feature(soup, 'Ambiance de travail'),
            'formation': self.extract_feature(soup, 'Formation et développement des compétences'),
            'implication_employes': self.extract_feature(soup, 'Implication des employés'),
            'interet_projets': self.extract_feature(soup, 'Intérêts de projets / missions'),
            'valeur_culture': self.extract_feature(soup, 'Valeur et culture'),
            'evolution_crriere': self.extract_feature(soup, 'Evolution de carrière mobilité'),
            'vision_strategie': self.extract_feature(soup, 'Vision / Stratégie'),
            'transparence_information': self.extract_feature(soup, 'Transparence de l\'information'),
            'esprit_equipe': self.extract_feature(soup, 'Esprit d\'équipe'),
            'creativite_innovation': self.extract_feature(soup, 'Créativité / Innovation'),
            'politique_management': self.extract_feature(soup, 'Politique de management'),
            'communication_interne': self.extract_feature(soup, 'Communication interne'),
            'stress_passion': self.extract_feature(soup, 'Stress / Pression')

        }

    def scrap(self):
        tmp_list = []
        for link in self.input_config['links']:
            tmp_list.append(self.query_company_infos(link))
        self.output_datasets['profiles'] = self.output_datasets['profiles'].append(tmp_list, ignore_index=True)

    def quality_check(self):
        pass

